@@ -9,7 +9,6 @@ import collections
 import functools
 import http.server
 import itertools
-import logging
 import re
 import sys
 import zlib
@@ -18,6 +17,7 @@ from wsgiref.handlers import format_date_time
 import aiohttp
 from aiohttp import errors
 from aiohttp import multidict
+from aiohttp.log import internal_log
 
 METHRE = re.compile('[A-Z0-9$-_.]+')
 VERSRE = re.compile('HTTP/(\d+).(\d+)')
@@ -253,11 +253,13 @@ class HttpResponseParser(HttpParser):
 
 class HttpPayloadParser:
 
-    def __init__(self, message, length=None, compression=True, readall=False):
+    def __init__(self, message, length=None, compression=True, readall=False,
+                 response_with_body=True):
         self.message = message
         self.length = length
         self.compression = compression
         self.readall = readall
+        self.response_with_body = response_with_body
 
     def __call__(self, out, buf):
         # payload params
@@ -270,7 +272,11 @@ class HttpPayloadParser:
             out = DeflateBuffer(out, self.message.compression)
 
         # payload parser
-        if 'chunked' in self.message.headers.get('TRANSFER-ENCODING', ''):
+        if not self.response_with_body:
+            # don't parse payload if it's not expected to be received
+            pass
+
+        elif 'chunked' in self.message.headers.get('TRANSFER-ENCODING', ''):
             yield from self.parse_chunked_payload(out, buf)
 
         elif length is not None:
@@ -287,7 +293,7 @@ class HttpPayloadParser:
             if self.readall and getattr(self.message, 'code', 0) != 204:
                 yield from self.parse_eof_payload(out, buf)
             elif getattr(self.message, 'method', None) in ('PUT', 'POST'):
-                logging.warn(  # pragma: no cover
+                internal_log.warning(  # pragma: no cover
                     'Content-Length or Transfer-Encoding header is required')
 
         out.feed_eof()
@@ -336,10 +342,10 @@ class HttpPayloadParser:
                 out.feed_data(chunk)
                 required -= len(chunk)
         except aiohttp.EofStream:
-            raise errors.IncompleteRead(required, length-required)
+            raise errors.IncompleteRead(length-required, required)
 
     def parse_eof_payload(self, out, buf):
-        """Read all bytes untile eof."""
+        """Read all bytes until eof."""
         try:
             while True:
                 out.feed_data((yield from buf.readsome()))
@@ -348,7 +354,7 @@ class HttpPayloadParser:
 
 
 class DeflateBuffer:
-    """DeflateStream decomress stream and feed data into specified stream."""
+    """DeflateStream decompress stream and feed data into specified stream."""
 
     def __init__(self, out, encoding):
         self.out = out
@@ -377,7 +383,7 @@ class DeflateBuffer:
 def wrap_payload_filter(func):
     """Wraps payload filter and piped filters.
 
-    Filter is a generatator that accepts arbitrary chunks of data,
+    Filter is a generator that accepts arbitrary chunks of data,
     modify data and emit new stream of data.
 
     For example we have stream of chunks: ['1', '2', '3', '4', '5'],
@@ -403,7 +409,7 @@ def wrap_payload_filter(func):
 
       1. If filter receives bytes object, it should process data
          and yield processed data then yield EOL_MARKER object.
-      2. If Filter recevied EOF_MARKER, it should yield remaining
+      2. If Filter received EOF_MARKER, it should yield remaining
          data (buffered) and then yield EOF_MARKER.
     """
     @functools.wraps(func)
@@ -433,8 +439,8 @@ def filter_pipe(filter, filter2):
       2. Reads yielded values from the first filter until it receives
          EOF_MARKER or EOL_MARKER.
       3. Each of this values is being send to second filter.
-      4. Reads yielded values from second filter until it recives EOF_MARKER or
-         EOL_MARKER. Each of this values yields to writer.
+      4. Reads yielded values from second filter until it receives EOF_MARKER
+         or EOL_MARKER. Each of this values yields to writer.
     """
     chunk = yield
 
@@ -487,7 +493,7 @@ class HttpMessage:
 
     Now we can use chunked writer to write stream to a network stream.
     First call to write() method sends response status line and headers,
-    add_header() and add_headers() method unavailble at this stage:
+    add_header() and add_headers() method unavailable at this stage:
 
     >> with open('...', 'rb') as f:
     ..     chunk = fp.read(8196)
@@ -500,7 +506,7 @@ class HttpMessage:
 
     writer = None
 
-    # 'filter' is being used for altering write() bahaviour,
+    # 'filter' is being used for altering write() behaviour,
     # add_chunking_filter adds deflate/gzip compression and
     # add_compression_filter splits incoming data into a chunks.
     filter = None
@@ -591,7 +597,7 @@ class HttpMessage:
             if name == 'USER-AGENT':
                 self._has_user_agent = True
 
-            # ignore hopbyhop headers
+            # ignore hop-by-hop headers
             self.headers.add(name, value)
 
     def add_headers(self, *headers):
@@ -651,7 +657,7 @@ class HttpMessage:
         self.headers['CONNECTION'] = connection
 
     def write(self, chunk):
-        """write() writes chunk of data to a steram by using different writers.
+        """write() writes chunk of data to a stream by using different writers.
         writer uses filter to modify chunk of data. write_eof() indicates
         end of stream. writer can't be used after write_eof() method
         being called. write() return drain future.
