@@ -346,10 +346,38 @@ class ProxyConnectorTests(unittest.TestCase):
 
         ClientRequestMock.assert_called_with(
             'GET', 'http://proxy.example.com',
-            auth=aiohttp.BasicAuth(None, None),
+            auth=None,
             headers={'Host': 'www.python.org'},
             loop=loop_mock)
 
+    def test_proxy_auth(self):
+        with self.assertRaisesRegex(AssertionError,
+                                    "must be None or BasicAuth"):
+            aiohttp.ProxyConnector('http://proxy.example.com',
+                                   proxy_auth=('user', 'pass'),
+                                   loop=unittest.mock.Mock())
+
+    @unittest.mock.patch('aiohttp.connector.ClientRequest')
+    def test_proxy_override_auth(self, ClientRequestMock):
+        req = ClientRequest('GET', 'http://www.python.org')
+
+        loop_mock = unittest.mock.Mock()
+        tr, proto = unittest.mock.Mock(), unittest.mock.Mock()
+        self._fake_coroutine(loop_mock.create_connection, (tr, proto))
+
+        connector = aiohttp.ProxyConnector(
+            'http://user:pass@proxy.example.com',
+            proxy_auth=aiohttp.BasicAuth(None, None),
+            loop=loop_mock)
+
+        def check_proxy_req(*args, **kw):
+            proxy_req = ClientRequest(*args, **kw)
+            self.assertNotIn('AUTHORIZATION', proxy_req.headers)
+            return proxy_req
+
+        ClientRequestMock.side_effect = check_proxy_req
+        self.loop.run_until_complete(connector._create_connection(req))
+
     def test_proxy_connection_error(self):
         connector = aiohttp.ProxyConnector('http://proxy.example.com',
                                            loop=self.loop)
@@ -374,7 +402,7 @@ class ProxyConnectorTests(unittest.TestCase):
         loop_mock = unittest.mock.Mock()
         connector = aiohttp.ProxyConnector(
             'http://proxy.example.com', loop=loop_mock,
-            proxy_login='user', proxy_passwd='pass')
+            proxy_auth=aiohttp.BasicAuth('user', 'pass'))
         connector._resolve_host = resolve_mock = unittest.mock.Mock()
         self._fake_coroutine(resolve_mock, [unittest.mock.MagicMock()])
 
@@ -392,6 +420,11 @@ class ProxyConnectorTests(unittest.TestCase):
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
+        ClientRequestMock.assert_called_with(
+            'GET', 'http://proxy.example.com',
+            auth=aiohttp.BasicAuth('user', 'pass'),
+            loop=unittest.mock.ANY, headers=unittest.mock.ANY)
+
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_auth_from_url(self, ClientRequestMock):
         proxy_req = ClientRequest('GET', 'http://user:pass@proxy.example.com')
@@ -419,6 +452,10 @@ class ProxyConnectorTests(unittest.TestCase):
         self.assertNotIn('AUTHORIZATION', proxy_req.headers)
         self.assertNotIn('PROXY-AUTHORIZATION', proxy_req.headers)
 
+        ClientRequestMock.assert_called_with(
+            'GET', 'http://user:pass@proxy.example.com',
+            auth=None, loop=unittest.mock.ANY, headers=unittest.mock.ANY)
+
     @unittest.mock.patch('aiohttp.connector.ClientRequest')
     def test_auth__not_modifying_request(self, ClientRequestMock):
         proxy_req = ClientRequest('GET', 'http://user:pass@proxy.example.com')
